
//
// rmst_source.hh  : RmstSource Class
// authors         : Fred Stann
//
// Copyright (C) 2003 by the University of Southern California
// $Id: security_app.hh,v 1.2 2004/03/14 23:48:43 rajgarhi Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#ifndef RMST_SRC
#define RMST_SRC

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif //HAVE_CONFIG_H
#include<set>
#include<map>
#include<algorithm>
#include "diffapp.hh"
#define PAYLOAD_SIZE 100
#include "/usr/include/jpeglib.h"
#include <sys/types.h>
#include <sys/socket.h>

class SecurityApp;
typedef struct events
{
  int size;
  unsigned char *frame;
  handle h;
};
typedef struct querySource
{
  int x;
  int y;
};

class Response_timer:public TimerCallback
{
public:
  SecurityApp *source_app;
  int q,size,fx,fy;
  int timessent;
  unsigned char *blob;
  Response_timer(SecurityApp *,int,int,int,unsigned char*,int);
  int expire();
};
typedef struct respData
{
  int size;
  unsigned char * respBlob;
  Response_timer *callback_object;
};
typedef std::map<int,NR::handle,std::less<int > > queryHandles;     //publication handles used to send response to queries
typedef std::map<int,bool,std::less<int > > queryMap;      // all the queries seen
typedef std::map<int, events, std::greater<int > > eventMap;  // all the events seen
typedef std::map<int,querySource, std::less<int> > response_state_info;
typedef std::map<int, respData,std::less<int> > responseMap; // map queryids to the last response blob sent 
class RmstSrcReceive : public NR::Callback {
public:
  RmstSrcReceive(SecurityApp *rmstSrc) {src_ = rmstSrc; }
  void recv(NRAttrVec *data, NR::handle my_handle);
protected:
  SecurityApp *src_;
};
class Subscriptionint_refresh:public TimerCallback
{
public:
  SecurityApp *source_app;
  Subscriptionint_refresh(SecurityApp *);
  int expire();
};

class CameraMaintainer
{
 public:
   unsigned char* jpgBuf;
  int jpgSize;
  bool initCamera();
  void CameraMaintainer::convertTgaToJpg (char *image);
  char* pullCamera();
  
  int height,width,size,palette,videoFD;
  ~CameraMaintainer();
};
class RmstStart:public TimerCallback
{
public:
  SecurityApp *src;
  int timage;
  RmstStart(int, SecurityApp*);
  int expire();
};
class CameraPuller:public TimerCallback
{
public:
  CameraMaintainer *cam_obj;
  CameraPuller(CameraMaintainer *);
  int expire();
};
   
class SecurityApp : public DiffApp {
public:
  handle imagepub;
  int unmarshalX(unsigned char *);
  int unmarshalY(unsigned char *);
  int unmarshalT(unsigned char *);
  void marshal(int,int,int,unsigned char *);
  void setupImagePublication();
  void sendImage(int);
  struct sockaddr_in sock;
  unsigned short int myPort;
  int sockfd;
handle setupResponsePublication();
  void publishPicAttrs(int);
  queryHandles query_handles;
  responseMap responses_seen;
  queryMap queries_seen;
  eventMap events_seen;
  response_state_info resp_info;   
  SecurityApp(int argc, char **argv,int x,int y,int c);
  int xcor,ycor;
  virtual ~SecurityApp(){};
  bool fsrc;
  bool have_camera;
  void initiateQuery(int);
  void run();
  void recv(Message *msg, handle h) { } 
  NR* getDr() {return dr_;} 
  void eventDetect(int);
  void addtimer(int t,TimerCallback *tc);
  int num_subscriptions_; 
  unsigned short int no;
  void processQuery(NRAttrVec *);
  void processResponse(NRAttrVec *);
  bool exists(unsigned char *,int,unsigned char *);
  void sendResponse(int,int,int,unsigned char *,int);
  void setupResponseSubscriptionInterest();
  int lasteventSeen;
protected:
  RmstSrcReceive *mr;
  handle subscription_interest_handle_;
  void setupResponseSubscription(int,int,int);

  

  handle responses_handle;
  int ck_val_;
  unsigned short int no_query;
   CameraMaintainer *camera_object;

 };
enum keys
  {
    SRCdata_Key = 3000,
    X_Coordinate_Key = 3001,
    Y_Coordinate_Key = 3002,
    Timestamph_Key = 3003,
    Picflag_Key = 3004,
    Query_Id_Key = 3005,
    Timestampl_Key = 3006,
    SrcX_Key = 3007,
    SrcY_Key = 3008,
    FIRST_SRCX_Key = 3009,
    FIRST_SRCY_Key = 3010,
    EX_Coordinate_Key = 3011,
    EY_Coordinate_Key = 3012,
    nhops_Key = 3013
  };


extern NRSimpleAttributeFactory<int> nhops;
extern NRSimpleAttributeFactory<int> FIRST_SRCX;
extern NRSimpleAttributeFactory<int> FIRST_SRCY;
extern NRSimpleAttributeFactory<void *> SRCdata;
extern NRSimpleAttributeFactory<int> X;
extern NRSimpleAttributeFactory<int> Y;
extern NRSimpleAttributeFactory<int> EX;
extern NRSimpleAttributeFactory<int> EY;
extern NRSimpleAttributeFactory<int> queryid;
extern NRSimpleAttributeFactory<int> timestamp;
extern NRSimpleAttributeFactory<int> SRCX;
extern NRSimpleAttributeFactory<int> SRCY;
extern NRSimpleAttributeFactory<int> picflag;


#endif //RMST_SRC
