//
// srcrt.hh       : GPSR Filter include file
// author         : Xi Wang
// note           : Contains code pieces from - 
//                  Source Routing Filter 
//                  GEAR Filter
//
// Copyright (C) 2000-2003 by the Unversity of Southern California
// $Id: gpsr.hh,v 1.3 2005/05/30 01:29:57 fstann Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#ifndef GPSR_HH
#define GPSR_HH

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

//#include "gpsr_attr.hh"

#ifdef NS_DIFFUSION
#include "diffagent.h"
#endif // NS_DIFFUSION

#include "diffapp.hh"

#define GPSR_TARGETERR 0.001

#define GPSR_MAXHOP 1000000

#ifdef NS_DIFFUSION
 #define GPSR_BEACON_INTERVAL1 400
 #define GPSR_BEACON_INTERVAL2 60 * 1000
 #define GPSR_BEACON_RANDRATIO 0.1
 #define GPSR_BEACON_P1REP 2
#else 
 #define GPSR_BEACON_INTERVAL1 5 * 1000
 #define GPSR_BEACON_INTERVAL2 60 * 1000
 #define GPSR_BEACON_RANDRATIO 0.1
 #define GPSR_BEACON_P1REP 10
#endif

#define GPSR_NEIGHBOR_TTL 180 * 1000
#define GPSR_NTIMER_INTERVAL 35 * 1000

#define GPSR_SINK_TTL 200 * 1000
#define GPSR_SINKTIMER_INTERVAL 40 * 1000

#define GPSR_MSGBEACON     1000
#define GPSR_MSGDATA       1001

class GPSRNeighborEntry {
public:
  int32_t nid;
  int live;
  double location_x;
  double location_y;
  int ttl;
  int closer_pt(int myid, double myx, double myy, double ptx, double pty,
		int  ptipa, int  ptipb, double dstx, double dsty,
		double *closerx, double *closery);
};

typedef list<GPSRNeighborEntry *> GPSRNeighborListTempD;

class GPSRNeighborList : public GPSRNeighborListTempD {
public:
  GPSRNeighborEntry * ent_findbyid(int nid);
  GPSRNeighborEntry * ent_findshortest(double myx, double myy,
				       double tx, double ty);
  GPSRNeighborEntry * ent_next_ccw(double basebrg, double x,
				   double y, int p, GPSRNeighborEntry *inne = 0);
  GPSRNeighborEntry * ent_next_ccw(double myx, double myy,
				   GPSRNeighborEntry *inne, int p);
 // screen one edge
  void planarize_one(GPSRNeighborEntry *nep, int algo, double x, double y);
 // remove all crossing edges
  void planarize(int algo, double x, double y);
};

struct GPSRHeader {
public:
  int32_t mode;
  uint32_t hopcount;
  int32_t perihead[2];
  double enterperi_x, enterperi_y;

  uint32_t deliverymode;
  int32_t cnode;
  double cdist;

  void Unpack(void *p) {
    int32_t ti, tui;
    double td;
    char *pChar = (char*)p;
    memcpy(&ti, p, sizeof(ti)); pChar+=sizeof(ti); p=(void*)pChar; mode = ntohl(ti);
    memcpy(&tui, p, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar; hopcount = ntohl(tui);
    memcpy(&tui, p, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar; perihead[0] = ntohl(tui);
    memcpy(&tui, p, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar; perihead[1] = ntohl(tui);
    memcpy(&td, p, sizeof(td)); pChar+=sizeof(td); p=(void*)pChar; enterperi_x = td;
    memcpy(&td, p, sizeof(td)); pChar+=sizeof(td); p=(void*)pChar; enterperi_y = td;
    memcpy(&tui, p, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar; deliverymode = ntohl(tui);
    memcpy(&tui, p, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar; cnode = ntohl(tui);
    memcpy(&td, p, sizeof(td)); pChar+=sizeof(td); p=(void*)pChar; cdist = td;
  }

  void Pack(void *p) {
    int32_t ti, tui;
    double td;
    char *pChar = (char*)p;
    ti = htonl(mode); memcpy(p, &ti, sizeof(ti)); pChar+=sizeof(ti); p=(void*)pChar;
    tui = htonl(hopcount); memcpy(p, &tui, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar;
    tui = htonl(perihead[0]); memcpy(p, &tui, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar;
    tui = htonl(perihead[1]); memcpy(p, &tui, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar;
    td = enterperi_x; memcpy(p, &td, sizeof(td)); pChar+=sizeof(td); p=(void*)pChar;
    td = enterperi_y; memcpy(p, &td, sizeof(td)); pChar+=sizeof(td); p=(void*)pChar;
    tui = htonl(deliverymode); memcpy(p, &tui, sizeof(tui)); pChar+=sizeof(tui);  p=(void*)pChar;
    tui = htonl(cnode); memcpy(p, &tui, sizeof(tui)); pChar+=sizeof(tui); p=(void*)pChar;
    td = cdist; memcpy(p, &td, sizeof(td)); pChar+=sizeof(td); p=(void*)pChar;
  }

  int Size() {
    return 4*6+8*3;
  }
};

class GPSRSinkEntry {
public:
  u_int16_t port;
  int ttl;
  NRAttrVec *attrs;
};

class GPSRFilter;

class GPSRFilterReceive : public FilterCallback {
public:
  GPSRFilterReceive(GPSRFilter *app) : app_(app) {};
  void recv(Message *msg, handle h);

  GPSRFilter *app_;
};

typedef list<GPSRSinkEntry *> GPSRSinkList;

class GPSRBeaconTimer;
class GPSRTestTimer;
class GPSRNExpireTimer;
class GPSRSinkExpireTimer;

class GPSRFilter : public DiffApp {
public:
#ifdef NS_DIFFUSION
  GPSRFilter(int id);
  int command(int argc, const char*const* argv);
#else
  GPSRFilter(int argc, char **argv);
#endif // NS_DIFFUSION

  virtual ~GPSRFilter(){};
  void run();
  void recv(Message *msg, handle h);

protected:
  // General Variables
  handle filter_handle;

  // Receive Callback for the filter
  GPSRFilterReceive *filter_callback_;

  // Filter interface
  handle setupFilter();

  // Message Processing
  Message * ProcessMessage(Message *msg);

public:
  GPSRBeaconTimer * beacon_timer;
  GPSRNExpireTimer * nexpire_timer;
  GPSRSinkExpireTimer * sinkexpire_timer;
  GPSRTestTimer * test_timer;
protected:
  double location_x, location_y;
  double testdest_x, testdest_y;
  int pkt_count_;
  int rdm_id_;
  GPSRNeighborList neighbors;
  GPSRSinkList sink_list;
  int beacon_timer_count;
  int TestInterval;
  int32_t my_id_;
  int use_planar_;
  int log_en;

public:
#ifndef NS_DIFFUSION
  void parseCommandLine(int argc, char **argv);
#endif

  int BeaconExpire();
  int SinkExpire();
  int TestSendExpire();
  int NeighborExpire();

protected:
  GPSRNeighborEntry * findNeighbor(int32_t neighbor_id);
  void updateNeighbor(int32_t neighbor_id, double location_x, double location_y);
  double GeoDist(double x1, double y1, double x2, double y2);
  GPSRNeighborEntry * GreedyFind(double target_x, double target_y);
  GPSRNeighborEntry * Routing(GPSRHeader *gpsrh, double tlocx, double tlocy,
			      int lasthop, int &reason);
  void DataToSink(Message *msg);
  int FindLoc(NRAttrVec *attrlist, double & x, double & y);
  void Log(int type, GPSRHeader &hdr);
};

class GPSRBeaconTimer : public TimerCallback {
public:
  GPSRBeaconTimer(GPSRFilter *gpsr_class) : gpsr_class(gpsr_class) {};
  ~GPSRBeaconTimer() {};

  int expire() {
    return gpsr_class->BeaconExpire();
  }

  GPSRFilter *gpsr_class;
};


class GPSRNExpireTimer : public TimerCallback {
public:
  GPSRNExpireTimer(GPSRFilter *gpsr_class) : gpsr_class(gpsr_class) {};
  int expire() {
    return gpsr_class->NeighborExpire();
  }
  GPSRFilter *gpsr_class;
};


class GPSRSinkExpireTimer : public TimerCallback {
public:
  GPSRSinkExpireTimer(GPSRFilter *gpsr_class) : gpsr_class(gpsr_class) {};
  int expire() {
    return gpsr_class->SinkExpire();
  }
  GPSRFilter *gpsr_class;
};


class GPSRTestTimer : public TimerCallback {
public:
  GPSRTestTimer(GPSRFilter *gpsr_class) : gpsr_class(gpsr_class) {};
  int expire() {
    return gpsr_class->TestSendExpire();
  }
  GPSRFilter *gpsr_class;
};

extern double distance(double x1, double y1,  double x2, double y2);
extern double bearing(double x1, double y1, double x2, double y2);
inline int cross_segment(double, double, double, double, double, double,
			 double, double, double * = 0, double * = 0);

#define GPSRH_DATA_GREEDY 0	// greedy mode data packet
#define GPSRH_DATA_PERI 1	// perimeter mode data packet

#define GPSRH_EXACTDELIVERY 0   // normal
#define GPSRH_CLOSEDELIVERY 1	// deliver to the closest known node
#define GPSRH_TOCLOSE       2	// going to closest known node

#define GPSR_ROUTING_NOEDGE 1
#define GPSR_ROUTING_CIRCLE 2
#define GPSR_ROUTING_HOPNOTFOUND 3
#define GPSR_ROUTING_UNKNOWNMODE 4

#define PLANARIZE_RNG 0
#define PLANARIZE_GABRIEL 1

#endif // GSSR_HH
