//
// emstar.hh     : Various relevant emstar include files put together
// Authors       : Compiled by Fabio Silva
//
// Copyright (C) 2000-2003 by the University of Southern California
// $Id: emstar_misc.hh,v 1.6 2005/07/22 18:32:47 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include <sys/types.h>
#include <inttypes.h>
#include <sys/ioctl.h>

// From local_types.h
typedef uint32_t node_id_t;
typedef uint32_t if_id_t;

typedef struct loc {
  float x, y, z;
} loc_t;

// From sync.h
typedef enum {
  UNKNOWN = 0,
  SYNTHETIC = 1,
  PROCESSOR = 2,
  CODEC = 3,
  MOTE = 4,
  ZERO = 5,
  DEBUG = 6,
  CODEC2 = 7
} comp_id_t;

typedef struct sync_id_t_s {
  node_id_t node;		/* ID of the node */
  comp_id_t comp:16;		/* component */
  int index:16;			/* for padding, and convenience */
} sync_id_t;

/* "remote timestamp" structure */
typedef struct remote_ts {
  sync_id_t timebase;
  struct timeval stamp;
  uint32_t sum_errors;
} remote_ts_t;

// From link.h
/*
 * Please use care when adding to the link_pkt_type list -- if someone
 * else is using the same number as you (and hasn't checked their code
 * in yet), you can see a conflict between your code and theirs.  You
 * might want to use a "random" number (up to 255) instead of the next
 * number in the sequence.
 */
typedef uint8_t link_pkt_type_t;

typedef enum link_pkt_enum {
  PKT_TYPE_ALL = 0,		/* do not use!! */
  PKT_TYPE_PING = 1,		/* ping request/response protocol */
  PKT_TYPE_HB = 2,		/* heartbeats for neighbor discovery */
  PKT_TYPE_FRAGD = 3,		/* fragmentation and reassembly */
  PKT_TYPE_FLOOD = 4,		/* flooding algorithm wrapper */
  PKT_TYPE_LINK_STATS = 5,      /* link stats wrapper */
  PKT_TYPE_DIFFUSION = 6,	/* directed diffusion */
  PKT_TYPE_STATESYNC = 7,	/* state sync service */

  PKT_TYPE_ASCENT = 20,         /* ascent control packets */
  PKT_TYPE_ASCENT_APP = 21,     /* ascent application packets (can this be a user type?) */
  PKT_TYPE_CONNTEST_DATA = 22,  /* connectivity data packets */
  PKT_TYPE_CONNTEST_CTL = 23,   /* connectivity control packets */
  PKT_TYPE_CONNTEST = 24,       /* connectivity data packets (old) */
  PKT_TYPE_LINK_STATS2 = 25,    /* new link stats wrapper */

  PKT_TYPE_GLS = 30,		/* Geo-Linkstate Routing */
  PKT_TYPE_FLOOD_GLS = 31,

  PKT_TYPE_TIMESYNC = 100,	/* time synchronization (syncd) */
  PKT_TYPE_GSYNC = 101,		/* global timesync flooding app */

  PKT_TYPE_TOS = 222,		/* tos  messages */
  PKT_TYPE_MAC_CTRL = 223,	/* MAC control messages (sync etc) */

  PKT_TYPE_USER0 = 224          /* user-defined types */
  /* DO NOT USE NUMBERS GREATER THAN 224!! */
} link_pkt_enum_t;

typedef struct link_sender_id {
  pid_t pid;
  void *sender_info;
} link_sender_id_t;

/*
 * This is the "universal" link interface header -- the header used to
 * speak to almost any kind of packet-transport interface in the
 * system.  The over-the-air format will be different depending on the
 * type of network; this is just the struct used to talk to the driver.
 */
typedef struct link_pkt {
  union {
    if_id_t id;			/* source interface address */
    loc_t loc;
  } src;
  union {
    if_id_t id;			/* destination interface address */
    loc_t loc;
  } dst;
  union {
    if_id_t id;
    loc_t loc;
  } prev_hop;

  /*
   * time this packet was received.  if_rcv_time is the raw timeval
   * generated by the network interface, e.g. the mote clock.
   * rcv_time is that time, after conversion (using the timesync
   * service) to a plain local-clock timeval.  If timesync is not
   * available, rcv_time is an inaccurate best guess - a
   * gettimeofday() at the time the packet arrived.
   */
  struct timeval rcv_time;
  remote_ts_t if_rcv_time;

  link_pkt_type_t type:8;	/* protocol number (8 bits) */
  uint8_t flags;		/* space for flags, currently unused */
  uint8_t max_hops;		/* max number of hops, if routed */
  uint8_t seqno;		/* sequence number, used by SMAC's linkstats */
  uint8_t numTx;		/* For SMAC's linkstats */
  uint8_t numRx;		/* For SMAC's linkstats */
  uint8_t state;		/* For SMAC's linkstats */
  float rnp;                    /* metric for routing using lqe */
  uint8_t ext_type;		/* 2nd type field, for tos/smac messages */
  uint8_t ext_group;	        /* for tos_group */
  uint8_t retval;		/* return value for reciepts */
  uint8_t rssi;                 /* RSSI value for this packet, if available */
  uint8_t link_index;           /* Index of link that this comes in from */
  link_sender_id_t sender;      /* identifies application sender for receipt */
  uint8_t data[0] __attribute__ ((aligned(4)));		/* pointer to the data that follows */
} link_pkt_t; // this SHOULD NOT be packed!!

#define LINK_BROADCAST ((if_id_t) 0xFFFFFFFF)

// From packet_client.h
#define PD_MAX_FILTER_LEN 128

typedef struct pd_filter {
  int len;
  char data[PD_MAX_FILTER_LEN];
} pd_filter_t;

/*
 *  ioctl() numbers
 */

#define PD_IOCTL_TYPE 0x8b

// query the queue to see how big the first element is
#define PD_GET_FRONT_SIZE  _IOR(PD_IOCTL_TYPE, 0x1, int)

// 0x2: not here
// 0x3: not here

// gets and sets the loopback mode
#define PD_SET_LOOP_MODE   _IOW(PD_IOCTL_TYPE, 0x4, int)
#define PD_GET_LOOP_MODE   _IOR(PD_IOCTL_TYPE, 0x4, int)

// gets and sets the generic filter
#define PD_SET_FILTER      _IOW(PD_IOCTL_TYPE, 0x5, pd_filter_t)
#define PD_GET_FILTER      _IOR(PD_IOCTL_TYPE, 0x5, pd_filter_t)

// gets and sets the qlen for the outgoing queue
#define PD_SET_OUTQ_LEN    _IOW(PD_IOCTL_TYPE, 0x6, int)
#define PD_GET_OUTQ_LEN    _IOR(PD_IOCTL_TYPE, 0x6, int)

// gets and sets the qlen for the incoming queue
#define PD_SET_INQ_LEN     _IOW(PD_IOCTL_TYPE, 0x7, int)
#define PD_GET_INQ_LEN     _IOR(PD_IOCTL_TYPE, 0x7, int)

// From status_client.h
#define STATUS_IOCTL_TYPE 0x5e

#define STATUS_SIZE     0x01    /* returns size of buffer */
#define STATUS_CLEAR    0x03    /* clears trigger */
#define STATUS_SETMODE  0x08    /* sets bits according to bits below */

#define STATUS_MODE_BINARY  1

#define PD_SET_BINARY_MODE  _IOW(STATUS_IOCTL_TYPE, STATUS_SETMODE, int)
